use crate::config::config::{REDIS_ADDR, STATIC_WS_PWD};
use crate::handles::handle_messages::handle_other_message;
use crate::handles::heartbeat::handle_heartbeat;
use crate::typing::used_typed::{Connection, ConnectionMap, TaskMap};
use crate::utils::json_utils::{make_common_resp, parse_message};
use crate::utils::utils::get_connection_params;
use futures::{SinkExt, StreamExt};
use lazy_static::lazy_static;
use redis::Client;
use redis::Commands;
use redis_pool::SingleRedisPool;
use std::collections::HashMap;
use std::sync::{Arc, Mutex};
use std::time::{Duration, Instant};
use tokio::sync::Mutex as AsyncMutex;
use tokio::time;
use tokio_tungstenite::accept_hdr_async;
use tungstenite::handshake::server::{Request, Response};
use tungstenite::{Error, Message};

lazy_static! {
    static ref REDIS_POOL: SingleRedisPool = {
        let client = Client::open(REDIS_ADDR).expect("Failed to connect to Redis");
        redis_pool::RedisPool::new(client, 16, Some(512))
    };
}

lazy_static! {
    static ref CONNECTIONS: ConnectionMap = Arc::new(AsyncMutex::new(HashMap::new()));
    static ref TASKS: TaskMap = Arc::new(Mutex::new(HashMap::new()));
}

// 关闭之前绑定的 WebSocket 连接并取消对应的任务
async fn close_existing_connection(from_id: &str) {
    let task_to_abort = {
        let mut tasks = TASKS.lock().unwrap();
        tasks.remove(from_id)
    };

    if let Some(task) = task_to_abort {
        task.abort();
    }

    let old_connection = {
        let mut connections = CONNECTIONS.lock().await;
        let already_done = connections.get(&from_id.to_string());
        println!(
            "关闭之前绑定的 WebSocket 连接: {} {:?}",
            from_id, already_done
        );
        connections.remove(from_id)
    };

    if let Some(mut old_connection) = old_connection {
        // 尝试优雅地关闭旧连接
        if let Err(e) = old_connection.sender.close().await {
            println!("关闭旧的 WebSocket 发送器时出错: {}", e);
        }
        println!("关闭旧的 WebSocket 连接: {}", from_id);
    }
}

// 更新 update_connected_redis 函数，使其返回 Result 类型
async fn update_connected_redis() -> Result<(), redis::RedisError> {
    let from_ids: Vec<String>;
    {
        let connections = CONNECTIONS.lock().await;
        from_ids = connections.keys().cloned().collect();
    }
    println!("当前全局连接映射中的 from_id 数量: {}", from_ids.len());

    let mut con = REDIS_POOL
        .get_connection()
        .expect("Failed to get Redis connection");

    // 先清空集合
    println!("开始清空 Redis 中的 connected 集合");
    if let Err(e) = con.del::<_, ()>("connected") {
        println!("清空 Redis 中的 connected 集合时出错: {}", e);
        return Err(e);
    }

    // 将 from_ids 一次性批量添加到集合中
    if !from_ids.is_empty() {
        println!("开始将 from_id 批量添加到 Redis 中的 connected 集合");
        if let Err(e) = con.sadd::<_, _, ()>("connected", from_ids.as_slice()) {
            println!(
                "将 from_id 批量添加到 Redis 中的 connected 集合时出错: {}",
                e
            );
            return Err(e);
        }
    }

    println!("成功更新 Redis 中的 connected 集合");
    Ok(())
}

// 提取出来的处理握手的函数
fn handle_handshake(
    req: &Request,
    must_existed_params: &[&str],
) -> Result<HashMap<String, String>, String> {
    println!("新客户端连接: {}", req.uri());
    let connection_params = match get_connection_params(req.uri().to_string()) {
        Ok(p) => p,
        Err(e) => {
            let error_msg = format!("缺少重要连接数据段: {}", e);
            println!("{}", error_msg);
            return Err(error_msg);
        }
    };

    let not_existed = must_existed_params
        .iter()
        .find(|param| !connection_params.contains_key(&param.to_string()));

    if let Some(param) = not_existed {
        let error_msg = format!("缺少重要连接数据段: {}", param);
        println!("{}", error_msg);
        return Err(error_msg);
    }

    if connection_params.get("wsPwd").unwrap() != (STATIC_WS_PWD) {
        println!("wsPwd不正确！");
        return Err("wsPwd不正确！".to_string());
    }

    Ok(connection_params)
}

async fn handle_connection_error(from_id: &str) {
    println!("开始处理用户id: {} 的连接错误", from_id);
    // 从全局连接映射中移除该连接
    let removed = {
        let mut connections = CONNECTIONS.lock().await;
        let result = connections.remove(from_id).is_some();
        drop(connections); // 提前释放锁
        result
    };

    println!(
        "是否成功从全局连接映射中移除用户id: {}，结果: {}",
        from_id, removed
    );

    // 更新 Redis 中的 connected 集合
    if removed {
        match update_connected_redis().await {
            Ok(_) => println!(
                "成功更新 Redis 中的 connected 集合，移除用户id: {}",
                from_id
            ),
            Err(e) => println!(
                "更新 Redis 中的 connected 集合时出错，用户id: {}，错误信息: {}",
                from_id, e
            ),
        }
    }

    println!("开始尝试关闭用户id: {} 的 redis 连接", from_id);

    // 取消对应的任务
    {
        let mut tasks = TASKS.lock().unwrap();
        match tasks.remove(from_id) {
            Some(task) => {
                task.abort();
                println!("成功取消用户id: {} 的任务", from_id);
            }
            None => println!("未在全局任务映射中找到用户id: {} 的任务", from_id),
        }
    }

    println!("断开与用户id: {} 的连接并完成清理操作", from_id);
}

pub(crate) async fn handle_client(stream: tokio::net::TcpStream) -> Result<(), Error> {
    let must_existed_params = ["deviceId", "fromId", "wsPwd"];
    let mut connection_params = None;

    let ws_stream = match accept_hdr_async(stream, |req: &Request, resp| {
        match handle_handshake(req, &must_existed_params) {
            Ok(params) => {
                connection_params = Some(params);
                Ok(resp)
            }
            Err(error_msg) => {
                println!("{}", error_msg);
                let error_resp = Response::builder()
                    .status(400)
                    .header("Content-Type", "text/plain")
                    .body(Some(error_msg))
                    .unwrap();
                Err(error_resp)
            }
        }
    })
    .await
    {
        Ok(ws) => ws,
        Err(e) => {
            println!("WebSocket握手失败: {}", e);
            return Ok(());
        }
    };

    let (sender, receiver) = ws_stream.split();

    if let Some(params) = connection_params {
        if let Some(from_id) = params.get("fromId") {
            let from_id = from_id.clone();
            let from_id_clone = from_id.clone(); // 新增：克隆一份 from_id 用于闭包

            // 检查 Redis 中是否已经存在该 fromId
            close_existing_connection(&from_id).await;

            // 将新连接添加到全局连接映射
            {
                let mut connections = CONNECTIONS.lock().await;
                connections.insert(from_id.clone(), Connection { sender, receiver });
            }

            // 更新 Redis 中的 connected 集合
            let _ = update_connected_redis().await;

            let task = tokio::spawn(async move {
                let mut last_heartbeat_time = Instant::now();
                loop {
                    let current_connection = {
                        let mut connections = CONNECTIONS.lock().await;
                        connections.remove(&from_id_clone) // 移除连接，将其从 map 中取出
                    };
                    if let Some(mut connection) = current_connection {
                        let (mut receiver_ref, mut sender_ref) =
                            (connection.receiver, connection.sender);
                        tokio::select! {
                            // 处理消息接收
                            maybe_msg = receiver_ref.next() => {
                                match maybe_msg {
                                    Some(Ok(msg)) => {
                                        if msg.is_text() {
                                            let text = msg.to_text().unwrap();
                                            match parse_message(text) {
                                                Ok(data) => {
                                                    match data.msg_type.as_str() {
                                                        "Heart" => {
                                                            println!("收到客户端心跳消息 {:?}", &data);
                                                            handle_heartbeat(&mut last_heartbeat_time);
                                                            if let Ok(json_str) = make_common_resp(Default::default(), "Heart") {
                                                                if let Err(e) = sender_ref.send(Message::text(json_str)).await {
                                                                    println!("发送心跳信息失败: {}", e);
                                                                    handle_connection_error(&from_id_clone).await;
                                                                    break;
                                                                }
                                                            }
                                                        },
                                                        _ => {
                                                            if let Err(e) = handle_other_message(&mut sender_ref, &data).await {
                                                                println!("Failed to handle other message: {}", e);
                                                                handle_connection_error(&from_id_clone).await;
                                                                break;
                                                            }
                                                        }
                                                    }
                                                }
                                                Err(e) => {
                                                    println!("解析JSON数据出错: {}", e);
                                                }
                                            }
                                        }
                                    }
                                    Some(Err(e)) => {
                                        println!("接受客户端消息出错: {}", e);
                                        handle_connection_error(&from_id_clone).await;
                                        break;
                                    }
                                    None => {
                                        println!("客户端断开连接");
                                        handle_connection_error(&from_id_clone).await;
                                        break;
                                    }
                                }
                            }
                            // 处理心跳超时
                            _ = time::sleep_until(tokio::time::Instant::from(last_heartbeat_time + Duration::from_secs(20))) => {
                                println!("用户id-{} 20秒内没有发送心跳，挂断连接", from_id_clone); // 使用克隆后的 from_id
                                break;
                            }
                        }
                        // 处理完一轮后，若连接还未断开，将其重新放回 CONNECTIONS
                        {
                            let mut connections = CONNECTIONS.lock().await;
                            connections.insert(
                                from_id_clone.clone(),
                                Connection {
                                    sender: sender_ref,
                                    receiver: receiver_ref,
                                },
                            );
                        }
                    } else {
                        break;
                    }
                }
                println!("断开与用户id: {}，连接", from_id_clone); // 使用克隆后的 from_id
                                                                   // 从全局连接映射中移除该连接
                {
                    let mut connections = CONNECTIONS.lock().await;
                    connections.remove(&from_id_clone); // 使用克隆后的 from_id
                }
                // 更新 Redis 中的 connected 集合
                let _ = update_connected_redis().await;
            });

            // 将任务句柄存储到全局任务映射中
            let mut tasks = TASKS.lock().unwrap();
            tasks.insert(from_id, task); // 使用原始的 from_id
        }
    } else {
        println!("无法获取连接参数");
    }

    Ok(())
}
